%numerical solution to Schroedinger equation for
%rectangular potential well with infinite barrier energy

length = 1;	                     %length of well (nm)
npoints=200;					 %number of sample points
x=linspace(0,length,npoints+1)';	 %position of sample points
num_sol=199;			             %number of solutions sought
dx = x(2)-x(1);

v = zeros(size(x));              %potential (eV)

[energy,phi,H]=schrod(length,npoints,v,num_sol);		%call Schrod.

hbar=1.054571596;              %Planck's constant (x10^34 J s)
echarge=1.602176462;		   %electron charge (x10^19 C)
emass=9.10938188;              %electron mass (x10^31 kg)

% compute theoretical values for energy levels of particle in a box
n = num_sol:-1:1; n = fliplr(n); % flip depending on what schrod returns 
Eth = n.^2*hbar^2*pi^2/(2*emass*length^2)/echarge;

% prepare some state
psi = zeros(size(x));
ind = x > 0.2 & x < 0.4; psi(ind) = sin((x(ind) - 0.2)/length*5*pi); psi = 1i*psi;

psi = psi/sqrt(psi'*psi); disp(['<psi|psi>=' num2str(psi'*psi)]); %normalize

c = phi'*psi; % decomposes psi into phi basis (phi = matrix)

xop = sparse(diag(x)); % x-operator
pop = diag(ones(npoints,1),1)-diag(ones(npoints,1),-1); % p-operator
pop = sparse(pop)/(2i*dx); %note p-op is actually k operator (k = p/hbar)

p2 = diag(ones(npoints,1),1)+diag(ones(npoints,1),-1)-2*diag(ones(npoints+1,1)); % p-operator
p2 = -sparse(p2)/(dx^2); %note p^2-op is actually k^2 operator

% print at most 10 first energy levels
if num_sol > 10, nval = 10; else nval = num_sol; end
for i=1:nval
    fprintf('eigenenergy (%d) = %f, theory = %f eV\n', i, energy(i), Eth(i));
end

figure(1); % display potential inside the box
plot(x,v,'b');xlabel('Distance (nm)'),ylabel('Potential energy, (eV)');

figure(2); % display comparison between theory and schrod.m
plot(n, Eth, '-', n, energy, 'o');
xlabel('eigenvalue #'); ylabel('energy (eV)'); legend('theory', 'schrod');

figure(3); % show probability density for psi
plot(x, abs(psi).^2, '.-'); ylabel('|\psi(x)|^2'); xlabel('Distance (nm)');

fprintf('checking decomposition norm: %g\n', c'*c);
fprintf('expectation energy: %g eV\n', c'*diag(energy)*c);
fprintf('expectation energy (another method): %g eV\n', psi(2:end-1)'*H*psi(2:end-1));
[v,i] = max(abs(c).^2);
fprintf('most probable outcome in energy measurement: energy(%g) = %g eV\n', i, energy(i));
fprintf('expectation position: %g nm\n', psi'*xop*psi);
fprintf('uncertainty Dx*Dp (in units of hbar): %g\n', sqrt((psi'*p2*psi)-(psi'*pop*psi)^2)*sqrt((psi'*xop^2*psi)-(psi'*xop*psi)^2));
