function [wx, paxis] = wig2(xaxis, psi)
%WIG2	 Computes the Wigner Distribution
%	[WX,PAXIS] = WIG2(XAXIS,PSI)
%   Inputs:
%     XAXIS - array of x values in nm
%	  PSI   - wavefunction, should be same sized vector as XAXIS
%   Outputs:
%	  WX    - the Wigner distribution
%             rows correspond to momentum
%             columns to positions
%	  PAXIS - momentum axis in x10^-31 kg nm/fs 
%
% Modified from wig2.m available as a part of HOSA package
%          by I.V. Bazarov, Sept 4, 2012

hbar=1.054571596;              %Planck's constant (x10^34 J s)

% --------------------- parameter checks --------------------------
if nargin ~= 2
   error('wig2: wrong number of input arguments');
end

[m, n] = size(psi);
if (min(m,n) ~= 1)
   disp(['wig2: input argument psi is a ',int2str(m),' by ',int2str(n), ...
         ' array'])
   error('Input argument psi must be a vector');
end

[m, n] = size(xaxis);
if (min(m,n) ~= 1)
   disp(['wig2: input argument axis is a ',int2str(m),' by ',int2str(n), ...
         ' array'])
   error('Input argument xaxis must be a vector');
end

if length(xaxis) ~= length(psi)
    error('wig2: xaxis and psi must be of the same length');
end

lx = length(psi);
lfft = 2^nextpow2(2*lx);                      % minimum FFT length

nfft = lfft;

if (nfft < 2*lx)
   disp('WIG2: FFT length must exceed twice the signal length')
   disp(['     resetting FFT length to ',int2str(lfft)])
   nfft = lfft;
end

x = zeros(nfft,1);   x(1:lx) = psi(:);          cx = conj(x);
wx = zeros(nfft,lx);
L1 = lx-1;

% ---- compute r(xprime,x) = cx(x-xprime/2) * x(x+xprime/2) ------

for n=0:L1
   indm = max(-n,-L1+n) : min(n,L1-n);
   indy = indm + (indm < 0) * nfft ;   % output indices y(m;n)
   y = zeros(nfft,1);
   y(indy + 1) = x(n+indm + 1) .* cx(n-indm + 1);
   wx(:,n+1)   = y;
end

% ----------- WD(p,x) = FT (xprime-->p) r(xprime,x) --------------
wx = fft(wx);
wx = real(wx.');                  % force it to be real


% ----------- get WDF ---------------------------------------------
%  note the frequency scaling by 2M
%  WD(k,x) =  X(2k,x),  where X(k,x) = FT ( r(xprime,x) )

dx = xaxis(2)-xaxis(1); % used for conversion of k and hbar*k

nfftby2 = nfft/2;
wx = wx(:,[nfftby2+1:nfft,1:nfftby2]).'/(pi*hbar);
paxis = (-nfftby2:nfftby2-1)/(2*nfft)*2*pi/dx*hbar;
